@@ -31,6 +31,7 @@
 # interpreted as representing official policies, either expressed
 # or implied, of GRNET S.A.
 
+from urllib2 import quote
 from threading import Thread
 from json import dumps, loads
 from time import time
@@ -111,9 +112,10 @@ class Client(object):
         self.base_url = base_url
         self.token = token
         self.headers = {}
-        self.DATE_FORMATS = ["%a %b %d %H:%M:%S %Y",
-            "%A, %d-%b-%y %H:%M:%S GMT",
-            "%a, %d %b %Y %H:%M:%S GMT"]
+        self.DATE_FORMATS = [
+            '%a %b %d %H:%M:%S %Y',
+            '%A, %d-%b-%y %H:%M:%S GMT',
+            '%a, %d %b %Y %H:%M:%S GMT']
         self.http_client = http_client
 
     def _init_thread_limit(self, limit=1):
@@ -123,8 +125,8 @@ class Client(object):
 
     def _watch_thread_limit(self, threadlist):
         recvlog.debug('# running threads: %s' % len(threadlist))
-        if self._elapsed_old > self._elapsed_new\
-        and self._thread_limit < self.POOL_SIZE:
+        if (self._elapsed_old > self._elapsed_new) and (
+                self._thread_limit < self.POOL_SIZE):
             self._thread_limit += 1
         elif self._elapsed_old < self._elapsed_new and self._thread_limit > 1:
             self._thread_limit -= 1
@@ -161,12 +163,13 @@ class Client(object):
     def set_default_header(self, name, value):
         self.http_client.headers.setdefault(name, value)
 
-    def request(self,
-        method,
-        path,
-        async_headers={},
-        async_params={},
-        **kwargs):
+    def request(
+            self,
+            method,
+            path,
+            async_headers={},
+            async_params={},
+            **kwargs):
         """In threaded/asynchronous requests, headers and params are not safe
         Therefore, the standard self.set_header/param system can be used only
         for headers and params that are common for all requests. All other
@@ -176,7 +179,6 @@ class Client(object):
         E.g. in most queries the 'X-Auth-Token' header might be the same for
         all, but the 'Range' header might be different from request to request.
         """
-
         try:
             success = kwargs.pop('success', 200)
 
@@ -187,13 +189,15 @@ class Client(object):
                 data = dumps(kwargs.pop('json'))
                 self.set_default_header('Content-Type', 'application/json')
             if data:
-                self.set_default_header('Content-Length', unicode(len(data)))
+                self.set_default_header('Content-Length', '%s' % len(data))
 
             sendlog.info('perform a %s @ %s', method, self.base_url)
 
-            self.http_client.url = self.base_url
-            self.http_client.path = path
-            r = self.http_client.perform_request(method,
+            self.http_client.url = self.base_url + (
+                '/' if (self.base_url and self.base_url[-1]) != '/' else '')
+            self.http_client.path = quote(path.encode('utf8'))
+            r = self.http_client.perform_request(
+                method,
                 data,
                 async_headers,
                 async_params)
@@ -223,8 +227,8 @@ class Client(object):
             errstr = '%s' % err
             if not errstr:
                 errstr = ('%s' % type(err))[7:-2]
-            raise ClientError('%s\n' % errstr,
-                status=getattr(err, 'status', 0) or getattr(err, 'errno', 0))
+            status = getattr(err, 'status', getattr(err, 'errno', 0))
+            raise ClientError('%s\n' % errstr, status=status)
 
         self.http_client.reset_headers()
         self.http_client.reset_params()
